<?php
/**
 * @package D3GB
 * @version 1.2
 */
/*
Plugin Name: D3GB
Plugin URI: d3gb.usal.es
Description: Displays genome browsers in WordPress
Author: David Barrios, Carlos Prieto
Version: 1.2
License: CC BY-NC-SA 4.0
*/

defined( 'ABSPATH' ) or die;

define('D3GB_DATA_DIR', WP_CONTENT_DIR.'/uploads/D3GB');

require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
require_once dirname( __FILE__ ) . "/includes/functions.php";

// function to create the DB / Options / Defaults					
function d3gb_options_install() {

    if ( ! current_user_can( 'activate_plugins' ) )
      return;
    $plugin = isset( $_REQUEST['plugin'] ) ? $_REQUEST['plugin'] : '';
    check_admin_referer( "activate-plugin_{$plugin}" );

    global $wpdb;
    $db_name = $wpdb->prefix . 'd3gb';
 
    // create the ECPT metabox database table
    $sql = $wpdb->prepare("show tables like %s",$db_name);
    if($wpdb->get_var($sql) != $db_name) {
      $sql = "CREATE TABLE $db_name (`ID` TINYINT UNSIGNED NOT NULL AUTO_INCREMENT, `name` text NOT NULL, `description` text NOT NULL, UNIQUE KEY (`ID`));";

      dbDelta($sql);
    }

    mkdir(D3GB_DATA_DIR);
}

// function to delete the DB / Options / Defaults
function d3gb_options_unninstall() {

    if ( ! current_user_can( 'activate_plugins' ) )
      return;
    $plugin = isset( $_REQUEST['plugin'] ) ? $_REQUEST['plugin'] : '';
    check_admin_referer( "deactivate-plugin_{$plugin}" );

    global $wpdb;
    $db_name = $wpdb->prefix . 'd3gb';

    $sql = $wpdb->prepare("show tables like %s;", $db_name."%");
    $tables = $wpdb->get_results($sql, ARRAY_N);
    foreach($tables as $table){
        $tablename = $table[0];
        $sql = "DROP TABLE $tablename;";
        $wpdb->query($sql);
    }

    $sql = "DROP TABLE IF EXISTS $db_name;";
    $wpdb->query($sql);

    d3gb_rmdir(D3GB_DATA_DIR);
}

// run the install scripts upon plugin activation
register_activation_hook(__FILE__,'d3gb_options_install');
register_deactivation_hook(__FILE__,'d3gb_options_unninstall');

// define list tables
require( dirname( __FILE__ ) . '/includes/track_list_table.php' );
require( dirname( __FILE__ ) . '/includes/list_table.php' );

// REGISTER
function d3gb_add_menu_items(){
    add_menu_page('WP Genome Browser', 'Genome Browser', 'activate_plugins', 'gb_list', 'd3gb_render_list_page');
    add_submenu_page('gb_list','Add New Item','Add New', 'activate_plugins', 'add_item', 'd3gb_render_add_item');
} add_action('admin_menu', 'd3gb_add_menu_items');


// RENDER
function d3gb_render_list_page(){
  if(current_user_can('edit_plugins')){
    //Create an instance of our package class...
    $gbListTable = new d3gb_list_table();
    //Fetch, prepare, sort, and filter our data...
    $gbListTable->prepare_items();

    if(isset($_GET['message']) && $_GET['message'] == "delete")
      d3gb_display_message("Item(s) deleted.");

    if( isset($_GET['action']) ) {
      require_once dirname( __FILE__ ) . '/includes/actions.php';
      d3gb_handle_actions();
    }else{
      ?>
      <div class="wrap">
        
        <div id="icon-users" class="icon32"><br/></div>
        <h2>Genome Browsers <a href="<?php echo admin_url('admin.php?page=add_item'); ?>" class="page-title-action">Add New</a></h2>
        
        <!-- Forms are NOT created automatically, so you need to wrap the table in one to use features like bulk actions -->
        <form id="items-filter" method="get">
            <!-- For plugins, we also need to ensure that the form posts back to our current page -->
            <input type="hidden" name="page" value="<?php echo $_REQUEST['page'] ?>" />
            <!-- Now we can render the completed list table -->
            <?php $gbListTable->display() ?>
        </form>
        
      </div>
      <?php
    }
  }else
    d3gb_display_message("You must be administrator.",NULL,"error");
}

function d3gb_render_add_item(){
  if(current_user_can('edit_plugins')){
    $title = "Add Item";
    if ( isset( $_POST['add'] ) && ( $_POST['add'] == $title ) ) {

      check_admin_referer(sanitize_key($title));

      $name = isset($_POST['name'])?sanitize_text_field($_POST['name']):"";
      $description = isset($_POST['description'])?sanitize_text_field($_POST['description']):"";
      $post_assembly = isset($_POST['assembly'])?sanitize_file_name($_POST['assembly']):"";
      $post_fasta = isset($_POST['fasta'])?sanitize_file_name($_POST['fasta']):"";
      $post_genbank = isset($_POST['genbank'])?sanitize_file_name($_POST['genbank']):"";

      $assembly = array();
      if($_FILES['assembly']['name']!=""||$_FILES['fasta']['name']!=""||$post_assembly!=""||$post_fasta!=""||$_FILES['genbank']['name']!=""||$post_genbank!="") {
        global $wpdb;
        $db_name = $wpdb->prefix . 'd3gb';

        $wpdb->insert($db_name,array('name'=>$name,'description'=>$description));
        d3gb_display_message("Item added.","gb_list");

        $newid = $wpdb->insert_id;
        $tbl_tracks = $wpdb->prefix . 'd3gb_' . $newid . '_tracks';
        $tbl_segments = $wpdb->prefix . 'd3gb_' . $newid . '_segments';

        $sql = "CREATE TABLE $tbl_tracks (trackid TINYINT UNSIGNED NOT NULL AUTO_INCREMENT, trackname TEXT, type TEXT, color TEXT, scale TEXT, PRIMARY KEY (trackid));";
        dbDelta($sql);
        $sql = "CREATE TABLE $tbl_segments (rowid INT UNSIGNED NOT NULL AUTO_INCREMENT, trackid TINYINT UNSIGNED NOT NULL, chr TINYTEXT, start INT UNSIGNED, end INT UNSIGNED, name TINYTEXT, score TEXT, strand CHAR(1), thickStart INT UNSIGNED, thickEnd INT UNSIGNED, itemRGB TINYTEXT, blockCount SMALLINT UNSIGNED, blockSizes TEXT, blockStarts TEXT, PRIMARY KEY (rowid)) ENGINE=MyISAM;";
        dbDelta($sql);
        $sql = "CREATE INDEX location ON $tbl_segments (chr,start,end)";
        dbDelta($sql);

        if($_FILES['assembly']['name']!=""||$post_assembly!="") {
                $read = fopen($_FILES['assembly']['name']!="" ? $_FILES['assembly']['tmp_name'] : D3GB_DATA_DIR.'/'.$post_assembly, 'r');
                if($read){
                  while(!feof($read)) {
                    $row = explode("\t",trim(fgets($read),"\n"));
                    if(count($row)>=3){
                      foreach(array(1,2) as $i)
                        $row[$i] = intval($row[$i]);
                      foreach(array(3,4) as $i)
                        $row[$i] = isset($row[$i])?$row[$i]:'';

                      $assembly[] = $row;
                    }
                  }
                  fclose($read);
                }else{
                  d3gb_display_message("Cannot read assembly.",NULL,"error");
                }
        }

        if($_FILES['fasta']['name']!=""||$post_fasta!="") {
          $filename = $_FILES['fasta']['name']!="" ? $_FILES['fasta']['tmp_name'] : D3GB_DATA_DIR.'/'.$post_fasta;
          if(file_exists($filename)) {
            d3gb_load_fasta($filename,$newid);
            if(empty($assembly))
              $assembly = d3gb_load_assembly($newid);
          }else{
              d3gb_display_message("Cannot find fasta.",NULL,"error");
          }
        }

        if($_FILES['genbank']['name']!=""||$post_genbank!=""){
          $filename = $_FILES['genbank']['name']!="" ? $_FILES['genbank']['tmp_name'] : D3GB_DATA_DIR.'/'.$post_genbank;
          if(file_exists($filename)) {
            $assembly = d3gb_load_genbank($filename,$newid);
          }else{
            d3gb_display_message("Cannot read genbank.",NULL,"error");
          }
        }

        if(!empty($assembly)) {
          $wpdb->insert($tbl_tracks,array('trackname'=>'chromosomes','type'=>'chromosomes'));
          $trackid = $wpdb->insert_id;
          foreach($assembly as $row) {
              $wpdb->insert($tbl_segments,array('trackid'=>$trackid,'chr'=>$row[0],'start'=>$row[1],'end'=>$row[2],'name'=>$row[3],'score'=>$row[4]));
          }
        }
      }
      if(empty($assembly))
        d3gb_display_message("Assembly is missing.",NULL,"error");
    }
    d3gb_display_form("add",$title,array('name'=>'', 'description'=>'', 'assembly'=>'*file*', 'fasta'=>'*file*', 'genbank'=>'*file*'),sanitize_key($title));
  }else
    d3gb_display_message("You must be administrator.",NULL,"error");
}

// shortcode
function d3gb_display_gb( $atts ) {
  $a = shortcode_atts( array(
    'id' => 1,
  ), $atts );

  $gbID = intval($a['id']);

  if(isset($_GET['r'])){
    require_once dirname( __FILE__ ) . "/includes/get_sequence.php";
    d3gb_get_sequence($gbID);
  }else{
    require_once dirname( __FILE__ ) . "/includes/render_genome_browser.php";
    d3gb_render_gb($gbID);
  }
}

add_shortcode('genome_browser','d3gb_display_gb');

// ajax
require dirname( __FILE__ ) . "/includes/query.php";
add_action( 'wp_ajax_query', 'd3gb_query_callback' );
add_action( 'wp_ajax_nopriv_query', 'd3gb_query_callback' );

