<?php
defined( 'ABSPATH' ) or die;

function d3gb_handle_actions(){
  $action = sanitize_text_field($_GET['action']);
  $actions = array('edit','tracks','addtrack','edittrack','deletetrack');
  $key = isset($_GET['item'])?intval($_GET['item']):0;
  if ( in_array($action, $actions) && $key ) {

    global $wpdb;
    $tbl_name = $wpdb->prefix . 'd3gb';
    $tbl_tracks = $wpdb->prefix . 'd3gb_' . $key . '_tracks';
    $tbl_segments = $wpdb->prefix . 'd3gb_' . $key . '_segments';

    $name = isset($_POST['name'])?sanitize_text_field($_POST['name']):"";
    $description = isset($_POST['description'])?sanitize_text_field($_POST['description']):"";
    $type = isset($_POST['type'])?sanitize_text_field($_POST['type']):"";
    $types = array('gene','exons','domain','value','score','maptrack','vcf','gff');
    $type = in_array ($type, $types) ? $type : 'gene';
    $file = isset($_POST['file'])?sanitize_file_name($_POST['file']):"";
    $color = isset($_POST['color'])?sanitize_text_field($_POST['color']):"";
    $color = d3gb_validate_color($color);
    $scale = isset($_POST['scale'])?sanitize_text_field($_POST['scale']):"";
    $scale = preg_match('/^\[[0-9\.\-]+\,[0-9\.\-]+\]$/',$scale)?$scale:FALSE;
    $trackname = isset($_POST['trackname'])?sanitize_text_field($_POST['trackname']):"";

    // EDIT
            if( 'edit'==$action ) {
                $title = "Edit Item";
                if ( isset( $_POST['edit'] ) && ( $_POST['edit'] == $title ) ) {
                    check_admin_referer(sanitize_key($title).$key);
                    $wpdb->update($tbl_name,array('name'=>$name,'description'=>$description),array('ID'=>$key));
                    d3gb_display_message("Item edited.","gb_list");
                }
                $sql = $wpdb->prepare("SELECT name, description FROM $tbl_name WHERE ID=%d", $key);
                $values = $wpdb->get_row( $sql, ARRAY_A );
                d3gb_display_form("edit",$title,$values,sanitize_key($title).$key);
            }

    // View tracks
            if( 'tracks'==$action ) {
              $trackListTable = new d3gb_track_list_table($key);
              $trackListTable->prepare_items();
              $sql = $wpdb->prepare("SELECT name FROM $tbl_name WHERE ID=%d;", $key);
              ?>
                <div class="wrap">        
                  <div id="icon-users" class="icon32"><br/></div>
                  <h2><?php echo $wpdb->get_var($sql); ?> <a href="<?php echo admin_url('admin.php?page=gb_list&action=addtrack&item='.$key); ?>" class="page-title-action">Add Track</a></h2>
                  <form id="tracks-filter" method="get">
                    <input type="hidden" name="page" value="<?php echo $_REQUEST['page'] ?>" />
                    <input type="hidden" name="item" value="<?php echo $key ?>" />
                    <?php $trackListTable->display() ?>
                  </form>
                </div>
              <?php
            }

    // Add track
            if( 'addtrack'==$action ) {
                set_time_limit(0);
                $bed_fields = d3gb_get_bed_fields();
                $title = "Add Track";
                if ( isset( $_POST['track'] ) && ( $_POST['track'] == $title ) ) {

                  check_admin_referer(sanitize_key($title).$key);

                  $read = fopen($_FILES['file']['name']!="" ? $_FILES['file']['tmp_name'] : D3GB_DATA_DIR.'/'.$file, 'r');
                  if ($read) {

                    if($type == 'maptrack'){
                      $sql = $wpdb->prepare("SELECT trackid FROM $tbl_tracks WHERE type=%s;","maptrack");
                      $rmid = intval($wpdb->get_var($sql));
                      if($rmid>0){
                        $sql = $wpdb->prepare("DELETE FROM $tbl_tracks WHERE trackid = %d;", $rmid);
                        $wpdb->query($sql);
                        $sql = $wpdb->prepare("DELETE FROM $tbl_segments WHERE trackid = %d;", $rmid);
                        $wpdb->query($sql);
                      }
                    }

                   if($type == 'gff'){
                      $sql = "SELECT max(trackid) FROM $tbl_tracks";
                      $trackid = intval($wpdb->get_var($sql))+1;
                      $tracks = [];
                      while(!feof($read)) {
                        $line = explode("\t",trim(fgets($read),"\n"));
                        $fields = count($line);
                        if(substr($line[0],0,1)!='#' && $fields>=5 && $fields<=9){
                          $type = $line[2];
                          if($type!='exon'){
                            if(!in_array($type,array_keys($tracks)))
                              $tracks[$type] = array('trackid'=>$trackid+count($tracks), 'type'=>in_array($type,['gene','CDS'])?'gene':'domain', 'color'=>$color);
                            $insert_array = array('trackid'=>$tracks[$type]['trackid'],'score'=>array());
                            for($i=0;$i<$fields;$i++){
                              switch($i){
                                case 0:
                                  $insert_array['chr'] = $line[$i];
                                  break;
                                case 1:
                                  if($line[$i]!='.')
                                    $insert_array['score'][] = 'source='.$line[$i];
                                  break;
                                case 3:
                                  $insert_array['start'] = intval($line[$i])-1;
                                  break;
                                case 4:
                                  $insert_array['end'] = intval($line[$i]);
                                  break;
                                case 5:
                                  if($line[$i]!='.')
                                      $insert_array['score'][] = 'score='.$line[$i];
                                  break;
                                case 6:
                                  if(in_array($line[$i],['+','-']))
                                    $insert_array['strand'] = $line[$i];
                                  break;
                                case 7:
                                  if(in_array($line[$i],['1','2'])){
                                    if($insert_array['strand']!='-'){
                                      $insert_array['thickStart'] = $insert_array['start'] + intval($line[$i]);
                                      $insert_array['thickEnd'] = $insert_array['end'];
                                    }else{
                                      $insert_array['thickStart'] = $insert_array['start'];
                                      $insert_array['thickEnd'] = $insert_array['end'] - intval($line[$i]);
                                    }
                                  }
                                  break;
                                case 8:
                                  $attributes = explode(';',$line[$i]);
                                  foreach($attributes as $attr){
                                    if(preg_match('/^ID=/',$attr))
                                      $insert_array['name'] = str_replace('ID=','',$attr);
                                    else{
                                      try{
                                        $insert_array['score'][] = $attr;
                                      }catch(Exception $e){
                                        break;
                                      }
                                    }                                      
                                  }
                                  break;
                              }
                            }
                            if(empty($insert_array['score']))
                              unset($insert_array['score']);
                            else
                              $insert_array['score'] = implode('|',$insert_array['score']);
                            $wpdb->insert($tbl_segments,$insert_array);
                          }else{
                            $parent = preg_replace('/.*parent=|;.+/i','',$line[8]);
                            $start = intval($line[3])-1;
                            $end = intval($line[4]);
                            $sql = $wpdb->prepare("SELECT trackid,start,blockCount,blockSizes,blockStarts FROM $tbl_segments WHERE name=%s",$parent);
                            $block = $wpdb->get_row($sql,ARRAY_A);
                            $block['blockCount'] = intval($block['blockCount'])+1;
                            $block['blockSizes'] .= ($end-$start).',';
                            $block['blockStarts'] .= ($start-$block['start']).',';
                            unset($block['start']);
                            unset($block['trackid']);
                            $wpdb->update($tbl_segments,$block,array('name'=>$parent));
                          }
                        }
                      }
                      $sql = $wpdb->prepare("SELECT DISTINCT trackid FROM $tbl_segments WHERE blockCount>=%d",1);
                      $exons = $wpdb->get_col($sql);
                      foreach(array_keys($tracks) as $i){
                        if(in_array((string)$tracks[$i]['trackid'],$exons))
                          $tracks[$i]['type'] = 'exons';
                      }
                      d3gb_insert_tracks($tracks,$key);
                      d3gb_gene_aux($key);
                   }else{

                    $wpdb->insert($tbl_tracks,array('trackname'=>$name,'type'=>$type,'color'=>$color));
                    $trackid = $wpdb->insert_id;
                    while(!feof($read)) {
                      $insert_array = array('trackid'=>$trackid);
                      $line = explode("\t",trim(fgets($read),"\n"));
                      if( $type=='vcf') {
                        if(substr($line[0],0,14) == "##INFO=<ID=CSQ"){
                          $csq = str_replace(["##INFO=<ID=CSQ,Number=.,Type=String,Description=\"Consequence annotations from Ensembl VEP. Format: ","\">"],["",""],$line[0]);
                          $csq = explode("|",$csq);
                          $wpdb->update($tbl_tracks,array('scale'=>json_encode(array_slice($csq,0,8))),array('trackid'=>$trackid));
                        }else{

                        if(substr($line[0],0,2) != "##") {
                          if($line[0] == "#CHROM") {
                            for($i=9;$i<count($line);$i++)
                              $wpdb->insert($tbl_tracks,array('trackname'=>$line[$i],'type'=>'vcfsample'));
                          }else{
                            for($i=0;$i<5;$i++){
                              switch($i){
                                case 1:
                                  $val = intval($line[1])-1;
                                  break;
                                case 2:
                                  $val = intval($line[1]);
                                  break;
                                case 3:
                                  $val = $line[2];
                                  break;
                                case 4:
                                  $aux = "";
                                  if(count($line)>7){
                                    $aux = array();
                                    $info = explode(";",$line[7]);
                                    foreach($info as $inf){
                                      if(isset($csq) && substr($inf,0,4) == "CSQ="){
                                        $csq_list = explode("|",explode(",",substr($inf,4))[0]);
                                        for($j=0;$j<count($csq);$j++){
                                          if($csq_list[$j]!=""){
                                            array_push($aux,$csq[$j]."=".$csq_list[$j]);
                                          }
                                        }
                                      }else{
                                        array_push($aux,$inf);
                                      }
                                    }
                                    $aux = implode("|",$aux);
                                    $aux = "|".$aux;
                                  }
                                  $val = "REF=".$line[3].(($line[4]!=".")?"|ALT=".$line[4]:"").(($line[5]!=".")?"|QUAL=".$line[5]:"").$aux;
                                  break;
                                default:
                                  $val = $line[$i];
                              }
                              $insert_array[$bed_fields[$i]] = $val;
                            }
                            $wpdb->insert($tbl_segments,$insert_array);
                            for($i=9;$i<count($line);$i++){
                                $insert_array['trackid'] = $trackid+$i-8;
                                $insert_array['score'] = d3gb_vcf_combine_format($line[8],$line[$i]);
                                if($insert_array['score']!==false)
                                  $wpdb->insert($tbl_segments,$insert_array);
                            }
                          }
                        }

                        }
                      }else{
                        if(count($line)>=3){
                          if(count($line)>12)
                            $line = array_slice($line,0,12);
                          foreach($line as $i => $val){
                            switch($i){
                              case 1:
                              case 2:
                              case 6:
                              case 7:
                              case 9:
                                $insert_array[$bed_fields[$i]] = intval($val);
                                break;
                              case 5:
                                if(in_array($val,['+','-']))
                                  $insert_array[$bed_fields[$i]] = $val;
                                break;
                              default:
                                $insert_array[$bed_fields[$i]] = $val;
                            }
                          }
                          $wpdb->insert($tbl_segments,$insert_array);
                        }
                      }
                    }

                    if(in_array($type,["value","score","maptrack"])){
                        $sql = $wpdb->prepare("SELECT concat(\"[\",floor(min(cast(score AS DECIMAL(10,2)))),\",\",ceil(max(cast(score AS DECIMAL(10,2)))),\"]\") AS scale FROM $tbl_segments WHERE trackid=%d", $trackid);
                        $scale = $wpdb->get_var($sql);
                        $wpdb->update($tbl_tracks,array('scale'=>$scale),array('trackid'=>$trackid));
                    }

                    if(in_array($type,["gene","exons"])){
                      d3gb_gene_aux($key);
                    }
                   }
                   fclose($read);
                   d3gb_display_message("Track(s) added.","gb_list&action=tracks&item=".$key);
                  } else {
                    d3gb_display_message("Cannot read file.",NULL,"error");
                  }
                }
                $values = array('name'=>'','type'=>$types,'color'=>'','file'=>'*file*');
                d3gb_display_form("track",$title,$values,sanitize_key($title).$key,'&action=tracks&item='.$key);
            }

    // EDIT track
            if( 'edittrack'==$action ) {
                $track = intval($_GET['track']);
                $title = "Edit Track";
                if ( isset( $_POST['edit'] ) && ( $_POST['edit'] == $title ) ) {

                    check_admin_referer(sanitize_key($title).$key);

                    $values = array('trackname'=>$trackname,'color'=>$color);
                    if($scale)
                      $values['scale'] = $scale;
                    $wpdb->update($tbl_tracks,$values,array('trackid'=>$track));
                    d3gb_display_message("track edited.","gb_list&action=tracks&item=".$key);
                }
                $sql = $wpdb->prepare("SELECT trackname, type, color, scale FROM $tbl_tracks WHERE trackid=%d", $track);
                $values = $wpdb->get_row( $sql, ARRAY_A );
                if(in_array($values['type'],['value','score'])){
                  unset($values['type']);
                }else{
                  unset($values['type']);
                  unset($values['scale']);
                }
                d3gb_display_form("edit",$title,$values,sanitize_key($title).$key,'&action=tracks&item='.$key);
            }

    // DELETE track
            if( 'deletetrack'==$action ) {
              $tracks = $_GET['track'];

              if(!is_array($tracks))
                    $tracks = array($tracks);

              $tracks = array_map('intval',$tracks);

              if ( isset( $_GET['delete'] ) && ( $_GET['delete'] == "yes" ) ) {

                check_admin_referer('deletetrack'.implode('',$tracks));

                foreach($tracks as $track) {
                    $sql = $wpdb->prepare("DELETE FROM $tbl_tracks WHERE trackid = %d;",$track);
                    $wpdb->query($sql);
                    $sql = $wpdb->prepare("DELETE FROM $tbl_segments WHERE trackid = %d;",$track);
                    $wpdb->query($sql);
                }
    ?>
        <script>
          window.location.href = "<?php echo admin_url('admin.php?page=gb_list&action=tracks&item='.$key.'&message=delete'); ?>";
        </script>
    <?php
              } else {
                  $id = "";
                  foreach( $tracks as $track => $value )
                    $id .= "&track[$track]=$value";
                ?>
    <div class="wrap">
        <h2>Delete Item</h2>
        <p>Sure you want to delete <?php echo count($tracks); ?> item(s)?</p>
        <a class="button" href="<?php echo admin_url('admin.php?page=gb_list&action=tracks&item='.$key); ?>">cancel</a>
        <a class="button button-primary" href="<?php echo wp_nonce_url(admin_url('admin.php?page=gb_list&action=deletetrack&delete=yes&item='.$key.$id),'deletetrack'.implode('',$tracks)); ?>">confirm</a>
    </div>
                <?php
              }
            }
  }
}
