<?php
defined( 'ABSPATH' ) or die;

function d3gb_location_decode($r){
  $aux = explode(":",$r);
  $chr = $aux[0];
  $start = intval(explode("-",$aux[1])[0]);
  $end = intval(explode("-",$aux[1])[1]);
  return array( 'chr' => $chr, 'start' => $start, 'end' => $end );
}

function d3gb_get_bed_fields() {
  return array('chr', 'start', 'end', 'name', 'score', 'strand', 'thickStart', 'thickEnd', 'itemRGB', 'blockCount', 'blockSizes', 'blockStarts');
}

function d3gb_vcf_combine_format($format,$data) {
  $format = explode(":",$format);
  $data = explode(":",$data);
  if(count($format)==count($data)){
    for($i=0; $i<count($format); $i++) {
      $format[$i] .= "=".$data[$i];
    }
    return implode("|",$format);
  }else
    return false;
}

function d3gb_gene_aux($key) {
  global $wpdb;
  $tbl_tracks = $wpdb->prefix . 'd3gb_' . $key . '_tracks';
  $tbl_segments = $wpdb->prefix . 'd3gb_' . $key . '_segments';
  $aux_genes = $wpdb->prefix . 'd3gb_' . $key . '_genes';
  $sql = "DROP TABLE IF EXISTS $aux_genes;";
  $wpdb->query($sql);
  $sql = $wpdb->prepare("CREATE TABLE $aux_genes ENGINE=MyISAM AS SELECT @n := @n + 1 n, chr, start, end, name, score FROM ($tbl_segments NATURAL JOIN $tbl_tracks, (SELECT @n := 0) m) WHERE (type=%s OR type=%s)","gene","exons");
  $wpdb->query($sql);
}

function d3gb_load_fasta($filename,$id) {
  $dirpath = D3GB_DATA_DIR . '/' . $id;
  if(mkdir($dirpath)){
    mkdir($dirpath."/sequences");
    $temp = $dirpath."/temp";
    mkdir($temp);

    if(!move_uploaded_file($filename,"$temp/fasta"))
      if(!rename($filename,"$temp/fasta"))
        d3gb_display_message("Unable to handle the file.",NULL,"error");

    $finfo = finfo_open(FILEINFO_MIME_TYPE);
    if(finfo_file($finfo, "$temp/fasta") == "application/zip"){
      rename("$temp/fasta","$temp/fasta.zip");
      $zip = new ZipArchive;
      $res = $zip->open("$temp/fasta.zip");
      if ($res === TRUE) {
        $zip->extractTo($temp);
        $zip->close();
      }
      unlink("$temp/fasta.zip");
    }elseif(finfo_file($finfo, "$temp/fasta") == "application/x-gzip"){
      rename("$temp/fasta","$temp/fasta.tar.gz");
      $p = new PharData("$temp/fasta.tar.gz");
      $p->decompress();
      $phar = new PharData("$temp/fasta.tar");
      $phar->extractTo($temp);
      unlink("$temp/fasta.tar.gz");
      unlink("$temp/fasta.tar");
    }

    finfo_close($finfo);

    $files = array_diff(scandir($temp), array('.','..'));
    foreach ($files as $file) {
      $read = fopen("$temp/$file",'r');
      if($read){
        $write = NULL;
        while(!feof($read)) {
          $line = fgets($read);
          if(strrpos($line,">") === false){
            fwrite($write,trim($line));
          }else{
            if(!is_null($write))
              fclose($write);
            $scf = str_replace(">","",trim($line));
            $scf = explode("|",$scf);
            $scf = explode(" ",$scf[0]);
            $write = fopen($dirpath."/sequences/".$scf[0].".fa",'w');
            fwrite($write,">".$scf[0]."\n");
          }
        }
        fclose($write);
        fclose($read);
      }
    }
    d3gb_rmdir($temp);
  }else{
    d3gb_display_message("Needed writing permissions in ".D3GB_DATA_DIR." to store sequences.",NULL,"error");
  }
}

function d3gb_load_assembly($id) {
  $assembly = array();
  $dirpath = D3GB_DATA_DIR . '/' . $id . '/sequences';
  foreach(array_diff(scandir($dirpath), array('.','..')) as $file) {
    $read = fopen("$dirpath/$file",'r');
    $scf = str_replace(">","",trim(fgets($read)));
    fclose($read);
    $end = filesize("$dirpath/$file")-count($scf)-1;
    $line = array($scf,0,$end,"","");
    $assembly[] = $line;
  }
  return $assembly;
}

function d3gb_insert_segment(&$segment,$data,$key) {
  if($segment['trackid']!=0) {
    global $wpdb;
    $tbl_segments = $wpdb->prefix . 'd3gb_' . $key . '_segments';

    $data = explode('/',$data);

              $pos = str_replace(['<','>','(',')'],['','','',''],array_shift($data));
              $cont = 0;
              $pos = str_replace('complement','',$pos,$cont);
              if($cont!=0){
                $segment['strand'] = '-';
                $cont = 0;
              }else{
                $segment['strand'] = '+';
              }
              $pos = str_replace('join','',$pos,$cont);
              if($cont!=0){
                $pos = explode(',',$pos);
                $segment['blockCount'] = count($pos);
                $segment['blockSizes'] = '';
                $segment['blockStarts'] = '';
                for($i=0;$i<$segment['blockCount'];$i++) {
                  $p = explode('..',$pos[$i]);
                  $p[0] = intval($p[0]);
                  $p[1] = intval($p[1]);
                  if($i==0)
                    $segment['start'] = $segment['thickStart'] = $p[0]-1;
                  $segment['blockSizes'] .= $p[1] - ($p[0]-1) . ',';
                  $segment['blockStarts'] .= (($p[0]-1) - $segment['start']) . ',';
                  if($i+1==$segment['blockCount'])
                    $segment['end'] = $segment['thickEnd'] = $p[1];
                }
              }else{
                $pos = explode('..',$pos);
                $segment['start'] = intval($pos[0])-1;
                $segment['end'] = intval($pos[1]);
              }

    $items = count($data);
    for($i=0;$i<$items;$i++) {
      if(preg_match('/^locus_tag=|^gene=/',$data[$i])) {
        $segment['name'] = preg_replace('/locus_tag=|gene=|"/','',$data[$i]);
        unset($data[$i]);
      }elseif(preg_match('/^translation=/',$data[$i])){
        unset($data[$i]);
      }
    }
    $segment['score'] = implode('|',$data);
    $wpdb->insert($tbl_segments,$segment);
    $segment = array('trackid'=>0,'chr'=>$segment['chr'],'start'=>0,'end'=>0);
  }
}

function d3gb_insert_tracks($tracks,$key) {
  global $wpdb;
  $tbl_tracks = $wpdb->prefix . 'd3gb_' . $key . '_tracks';
  foreach($tracks as $name => $values){
    $track = array('trackid' => $values['trackid'], 'trackname' => $name, 'type' => $values['type'], 'color' => $values['color']);
    $wpdb->insert($tbl_tracks,$track);
  }
}

function d3gb_load_genbank($filename,$id) {
  $assembly = [];
  $dirpath = D3GB_DATA_DIR . '/' . $id;
  if(mkdir($dirpath)){
    mkdir($dirpath."/sequences");
    $read = fopen($filename,'r');
    if($read){
      $write = NULL;
      $current = 0;
      $track_name = '';
      $tracks = [];
      $segment = array('trackid'=>0,'chr'=>'','start'=>0,'end'=>0);
      $segment_data = '';
      while(!feof($read)) {
        $str_line = trim(fgets($read),"\n");
        $line = preg_split("/ +/",$str_line);
        if($line[0]=='LOCUS'){
          $segment['chr'] = $line[1];
          $assembly[] = [$segment['chr'],0,$line[2],'',''];
          $current = 1;
        }elseif($line[0]=='FEATURES'){
          $current = 2;
        }elseif($line[0]=='ORIGIN'){
          d3gb_insert_segment($segment,$segment_data,$id);
          $write = fopen($dirpath."/sequences/".$segment['chr'].".fa",'w');
          fwrite($write,">".$segment['chr']."\n");
          $current = 3;
        }elseif(preg_match('/^[A-Z]/',$line[0])){
          $current = 0;
        }else{
          if($current==2){
            if(substr($str_line,5,1)!=' '){
              d3gb_insert_segment($segment,$segment_data,$id);
              $track_name = $line[1];
              if(!in_array($track_name,array_keys($tracks)))
                $tracks[$track_name] = array('trackid'=>count($tracks)+2,'type'=>in_array($track_name,['CDS','gene'])?'gene':'domain','color'=>'black');
              $segment['trackid'] = $tracks[$track_name]['trackid'];
              $segment_data = $line[2];
              if(strpos($segment_data,'join')!==false)
                $tracks[$track_name]['type'] = 'exons';
            }else{
              array_shift($line);
              $line = implode(' ',$line);
              $segment_data .= $line;
            }
          }
          if($current==3){
            if($line[0]=='//'){
              fclose($write);
            }else{
              $seq = preg_replace('/[0-9\s]+/','',$str_line);
              fwrite($write,$seq);
            }
          }
        }
      }
      fclose($read);
      d3gb_insert_tracks($tracks,$id);
      d3gb_gene_aux($id);
    }
  }else{
    d3gb_display_message("Needed writing permissions in ".D3GB_DATA_DIR." to store sequences.",NULL,"error");
  }
  return $assembly;
}

function d3gb_display_message ($text, $page = NULL, $status = 'success') {
    ?>
    <div id="message" class="notice notice-<?php echo $status; ?> is-dismissible">
      <p>
        <?php echo $text; if(!is_null($page)){ echo ' <a href="' . admin_url('admin.php?page='.$page) . '">View all</a>'; }?>
      </p>
      <button type="button" class="notice-dismiss">
        <span class="screen-reader-text">Dismiss this notice.</span>
      </button>
    </div>
    <?php
}

function d3gb_display_form ($type, $title, $values, $nonce_action, $return_page = '') {
?>
    <div class="wrap">
        
        <h2><?php echo $title; ?></h2>
<script type="text/javascript">
function loading(){
  jQuery("img.loading").css("display","inline-block");
}
</script>
        <form onsubmit="loading();" id="<?php echo $type; ?>-item" method="post" enctype="multipart/form-data">
        <?php 
          wp_nonce_field($nonce_action);
          foreach($values as $name => $val){ 
            if ($val=="*file*") {
              echo "<p>".ucfirst($name)."<br><input type=\"file\" name=\"$name\" /><br>or upload a file in <span style=\"font-weight:bold\">wp-content/uploads/D3GB</span> named:<br><input type=\"text\" name=\"$name\" value=\"\" /></p>";
            } elseif (is_array($val)) {
              echo "<p>".ucfirst($name)."<br><select name=\"$name\" />";
              foreach ($val as $v) {
                echo "<option value=\"$v\">$v</option>";
              }
              echo "</select></p>";
            } else {
              echo "<p>".ucfirst($name)."<br><input type=\"text\" name=\"$name\" value=\"$val\" /></p>";
            }
           }
        ?>
            <p>
              <a class="button" href="<?php echo admin_url('admin.php?page=gb_list'.$return_page); ?>">cancel</a>
              <input value="<?php echo $title; ?>" name="<?php echo $type; ?>" class="button button-primary button-large" type="submit" />
              <img class="loading" style="margin: -4px 4px; display: none;" src="<?php echo admin_url("images/loading.gif"); ?>" />
            </p>
        </form>
        
    </div>
    <?php
}

function d3gb_rmdir($dir) {
  if(is_dir($dir)){
    $files = array_diff(scandir($dir), array('.','..'));
    foreach ($files as $file) {
      (is_dir("$dir/$file")) ? d3gb_rmdir("$dir/$file") : unlink("$dir/$file");
    }
    return rmdir($dir);
  }else{
    return false;
  }
}

function d3gb_validate_color($color, $named = TRUE) {
      /* Validates hex color, adding #-sign if not found. Checks for a Color Name first to prevent error if a name was entered (optional).
      *   $color: the color hex value stirng to Validates
      *   $named: (optional), set to 1 or TRUE to first test if a Named color was passed instead of a Hex value
      */
     
      if ($named) {
     
        $named = array('aliceblue', 'antiquewhite', 'aqua', 'aquamarine', 'azure', 'beige', 'bisque', 'black', 'blanchedalmond', 'blue', 'blueviolet', 'brown', 'burlywood', 'cadetblue', 'chartreuse', 'chocolate', 'coral', 'cornflowerblue', 'cornsilk', 'crimson', 'cyan', 'darkblue', 'darkcyan', 'darkgoldenrod', 'darkgray', 'darkgreen', 'darkkhaki', 'darkmagenta', 'darkolivegreen', 'darkorange', 'darkorchid', 'darkred', 'darksalmon', 'darkseagreen', 'darkslateblue', 'darkslategray', 'darkturquoise', 'darkviolet', 'deeppink', 'deepskyblue', 'dimgray', 'dodgerblue', 'firebrick', 'floralwhite', 'forestgreen', 'fuchsia', 'gainsboro', 'ghostwhite', 'gold', 'goldenrod', 'gray', 'green', 'greenyellow', 'honeydew', 'hotpink', 'indianred', 'indigo', 'ivory', 'khaki', 'lavender', 'lavenderblush', 'lawngreen', 'lemonchiffon', 'lightblue', 'lightcoral', 'lightcyan', 'lightgoldenrodyellow', 'lightgreen', 'lightgrey', 'lightpink', 'lightsalmon', 'lightseagreen', 'lightskyblue', 'lightslategray', 'lightsteelblue', 'lightyellow', 'lime', 'limegreen', 'linen', 'magenta', 'maroon', 'mediumaquamarine', 'mediumblue', 'mediumorchid', 'mediumpurple', 'mediumseagreen', 'mediumslateblue', 'mediumspringgreen', 'mediumturquoise', 'mediumvioletred', 'midnightblue', 'mintcream', 'mistyrose', 'moccasin', 'navajowhite', 'navy', 'oldlace', 'olive', 'olivedrab', 'orange', 'orangered', 'orchid', 'palegoldenrod', 'palegreen', 'paleturquoise', 'palevioletred', 'papayawhip', 'peachpuff', 'peru', 'pink', 'plum', 'powderblue', 'purple', 'red', 'rosybrown', 'royalblue', 'saddlebrown', 'salmon', 'sandybrown', 'seagreen', 'seashell', 'sienna', 'silver', 'skyblue', 'slateblue', 'slategray', 'snow', 'springgreen', 'steelblue', 'tan', 'teal', 'thistle', 'tomato', 'turquoise', 'violet', 'wheat', 'white', 'whitesmoke', 'yellow', 'yellowgreen');
     
        if (in_array(strtolower($color), $named)) {
          /* A color name was entered instead of a Hex Value, so just exit function */
          return $color;
        }
      }
     
      if (preg_match('/^#[a-f0-9]{6}$/i', $color)) {
        return $color;
      }else if (preg_match('/^[a-f0-9]{6}$/i', $color)) {
        return '#' . $color;
      }
      return "#000000";
}
