<?php
defined( 'ABSPATH' ) or die;

if(!class_exists('WP_List_Table')){
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class d3gb_list_table extends WP_List_Table {

    function __construct(){
        global $status, $page;
                
        //Set parent defaults
        parent::__construct( array(
            'singular'  => 'item',     //singular name of the listed records
            'plural'    => 'items',    //plural name of the listed records
            'ajax'      => false        //does this table support ajax?
        ) );
        
    }

    function column_default($item, $column_name){
        switch($column_name){
            case 'description':
            case 'shortcode':
                return $item[$column_name];
            default:
                return print_r($item,true); //Show the whole array for troubleshooting purposes
        }
    }

    function column_name($item){
        
        //Build row actions
        $actions = array(
            'tracks' => sprintf('<a href="?page=%s&action=%s&item=%s">View Tracks</a>',$_REQUEST['page'],'tracks',$item['ID']),
            'edit' => sprintf('<a href="?page=%s&action=%s&item=%s">Edit</a>',$_REQUEST['page'],'edit',$item['ID']),
            'delete' => sprintf('<a href="?page=%s&action=%s&item=%s">Delete</a>',$_REQUEST['page'],'delete',$item['ID']),
        );
        
        //Return the name contents
        return sprintf('%1$s <span style="color:silver">(id:%2$s)</span>%3$s',
            /*$1%s*/ $item['name'],
            /*$2%s*/ $item['ID'],
            /*$3%s*/ $this->row_actions($actions)
        );
    }

    function column_cb($item){
        return sprintf(
            '<input type="checkbox" name="%1$s[]" value="%2$s" />',
            /*$1%s*/ $this->_args['singular'],  //Let's simply repurpose the table's singular label ("item")
            /*$2%s*/ $item['ID']                //The value of the checkbox should be the record's id
        );
    }

    function get_columns(){
        $columns = array(
            'cb' => '<input type="checkbox" />', //Render a checkbox instead of text
            'name' => 'Name',
            'description' => 'Description',
            'shortcode' => 'Shortcode'
        );
        return $columns;
    }

    function get_sortable_columns() {
        $sortable_columns = array(
            'name' => array('name',false),     //true means it's already sorted
            'description' => array('description',false)
        );
        return $sortable_columns;
    }

    function get_bulk_actions() {
        $actions = array(
            'delete'    => 'Delete'
        );
        return $actions;
    }

    function process_bulk_action() {

        global $wpdb;
        $tbl_name = $wpdb->prefix . 'd3gb';

        if ( isset( $_GET['item'] ) && ! empty( $_GET['item'] ) ) {

            $keys = $_GET['item'];
            if(!is_array($keys))
                  $keys = array($keys);
            $keys = array_map('intval',$keys);

    // DELETE
            if( 'delete'===$this->current_action() ) {

              if ( isset( $_GET['delete'] ) && ( $_GET['delete'] == "yes" ) ) {

                check_admin_referer('deleteitem'.implode('',$keys));

                foreach($keys as $key) {
                    $sql = $wpdb->prepare("DELETE FROM $tbl_name WHERE ID = %d;",$key);
                    $wpdb->query($sql);
                    foreach(array('genes','segments','tracks') as $tbl){
                      $tablename = $tbl_name."_".$key."_".$tbl;
                      $sql = "DROP TABLE IF EXISTS $tablename;";
                      $wpdb->query($sql);
                    }
                    d3gb_rmdir(D3GB_DATA_DIR . '/' . $key);
                }
    ?>
        <script>
          window.location.href = "<?php echo admin_url('admin.php?page=gb_list&message=delete'); ?>";
        </script>
    <?php
              } else {
                  $id = "";
                  foreach( $keys as $key => $value )
                    $id .= "&item[$key]=$value";
                ?>
    <div class="wrap">
        <h2>Delete Item</h2>
        <p>Sure you want to delete <?php echo count($keys); ?> item(s)?</p>
        <a class="button" href="<?php echo admin_url('admin.php?page=gb_list'); ?>">cancel</a>
        <a class="button button-primary" href="<?php echo wp_nonce_url(admin_url('admin.php?page=gb_list&action=delete&delete=yes'.$id),'deleteitem'.implode('',$keys)); ?>">confirm</a>
    </div>
                <?php
              }
            }
        } 
    }

    function prepare_items() {
        global $wpdb;
        $tbl_name = $wpdb->prefix . 'd3gb';

        $per_page = 8;
        
        $columns = $this->get_columns();
        $hidden = array();
        $sortable = $this->get_sortable_columns();
        
        $this->_column_headers = array($columns, $hidden, $sortable);
        
        $this->process_bulk_action();

        $current_page = $this->get_pagenum();
        
        $orderby = (!empty($_REQUEST['orderby']) && $_REQUEST['orderby']=='description') ? 'description' : 'name'; //If no sort, default to name
        $order = (!empty($_REQUEST['order']) && $_REQUEST['order']=='desc') ? 'desc' : 'asc'; //If no order, default to asc

        $sql = $wpdb->prepare("SELECT ID, name, description, CONCAT(\"[genome_browser id=\", ID, \"]\") AS shortcode FROM $tbl_name ORDER BY $orderby $order LIMIT %d, %d;", (($current_page-1)*$per_page), $per_page);
        $data = $wpdb->get_results($sql, ARRAY_A);

        $sql = "SELECT count(*) FROM $tbl_name";
        $total_items = $wpdb->get_var($sql);

        $this->items = $data;

        $this->set_pagination_args( array(
            'total_items' => $total_items,  //WE have to calculate the total number of items
            'per_page'    => $per_page,  //WE have to determine how many items to show on a page
            'total_pages' => ceil($total_items/$per_page)  //WE have to calculate the total number of pages
        ) );
    }
}

