<?php
defined( 'ABSPATH' ) or die;

if(!class_exists('WP_List_Table')){
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class d3gb_track_list_table extends WP_List_Table {
    private $key;

    function __construct($key){
        global $status, $page;
        $this->key = $key;
                
        //Set parent defaults
        parent::__construct( array(
            'singular'  => 'track',     //singular name of the listed records
            'plural'    => 'tracks',    //plural name of the listed records
            'ajax'      => false        //does this table support ajax?
        ) );
        
    }

    function column_default($item, $column_name){
        switch($column_name){
            case 'type':
            case 'color':
            case 'scale':
                return $item[$column_name];
            default:
                return print_r($item,true); //Show the whole array for troubleshooting purposes
        }
    }

    function column_name($item){
        
        //Build row actions
        $actions = array(
            'edit' => sprintf('<a href="?page=%s&action=%s&item=%s&track=%s">Edit</a>',$_REQUEST['page'],'edittrack',$this->key,$item['trackid']),
            'delete' => sprintf('<a href="?page=%s&action=%s&item=%s&track=%s">Delete</a>',$_REQUEST['page'],'deletetrack',$this->key,$item['trackid']),
        );
        
        //Return the name contents
        return sprintf('%1$s <span style="color:silver">(id:%2$s)</span>%3$s',
            /*$1%s*/ $item['trackname'],
            /*$2%s*/ $item['trackid'],
            /*$3%s*/ $this->row_actions($actions)
        );
    }

    function column_cb($item){
        return sprintf(
            '<input type="checkbox" name="%1$s[]" value="%2$s" />',
            /*$1%s*/ $this->_args['singular'],  //Let's simply repurpose the table's singular label ("item")
            /*$2%s*/ $item['trackid']                //The value of the checkbox should be the record's id
        );
    }

    function get_columns(){
        $columns = array(
            'cb' => '<input type="checkbox" />', //Render a checkbox instead of text
            'name' => 'Name',
            'type' => 'Type',
            'color' => 'Color',
            'scale' => 'Scale'
        );
        return $columns;
    }

    function get_sortable_columns() {
        $sortable_columns = array(
            'name' => array('name',false),     //true means it's already sorted
            'type' => array('type',false)
        );
        return $sortable_columns;
    }

    function get_bulk_actions() {
        $actions = array(
            'deletetrack'    => 'Delete'
        );
        return $actions;
    }

    function prepare_items() {
        global $wpdb;
        $tbl_tracks = $wpdb->prefix . 'd3gb_' . $this->key . '_tracks';

        $per_page = 8;
        
        $columns = $this->get_columns();
        $hidden = array();
        $sortable = $this->get_sortable_columns();
        
        $this->_column_headers = array($columns, $hidden, $sortable);
        
        $this->process_bulk_action();

        $current_page = $this->get_pagenum();
        
        $orderby = (!empty($_REQUEST['orderby']) && $_REQUEST['orderby']=='type') ? 'type' : 'trackname'; //If no sort, default to name
        $order = (!empty($_REQUEST['order']) && $_REQUEST['order']=='desc') ? 'desc' : 'asc'; //If no order, default to asc

        $sql = $wpdb->prepare("SELECT trackid, trackname, type, color, scale FROM $tbl_tracks WHERE type != %s ORDER BY $orderby $order LIMIT %d, %d;", "chromosomes", ($current_page-1)*$per_page, $per_page);
        $data = $wpdb->get_results($sql, ARRAY_A);

        $sql = "SELECT count(*) FROM $tbl_tracks";
        $total_items = $wpdb->get_var($sql);

        $this->items = $data;

        $this->set_pagination_args( array(
            'total_items' => $total_items,  //WE have to calculate the total number of items
            'per_page'    => $per_page,  //WE have to determine how many items to show on a page
            'total_pages' => ceil($total_items/$per_page)  //WE have to calculate the total number of pages
        ) );
    }
}

