var chr,
    start,
    end;

var fixedBounds = false;

function displayGenomeBrowser(){
    d3.select('body').on("click", function(){
        d3.select("div.plot div.tooltip").remove();
        d3.select("div.plot select").remove();
      })
    if(/^(\?r=)([\_\.0-9a-zA-Z]+)\:[0-9]+\-[0-9]+$/.test(window.location.search)){
      displayTracks(window.location.search.substring(3));
    } else {
      genomemap();
    }
}

function searchDB(name,type,callback){
  d3.json(queryUrl)
	    .header("Content-type", "application/x-www-form-urlencoded")
	    .post("action=query&gb="+gbID+"&"+type+"="+encodeURI(name), function(error, json) {
    if (error) throw error;
		json = (json.length == 0)? false : json;
		callback(json);
             });
}

function gotoLocation(){
  var str = d3.select("input[name=r]").node().value,
  patt = new RegExp("^([\_\.0-9a-zA-Z]+)\:[0-9]+\-[0-9]+$");
  if(patt.test(str))
    displayTracks(str);
  else
    return false;
}

function searchGene(){
  var thiz = d3.select("input[name=r]").node(),
  str = thiz.value,
  patt = new RegExp("^[\_\.0-9A-Za-z\-]{2,}$");
  d3.select("select[name=r]").remove();
  if(patt.test(str)){
    searchDB(str, "genes", function(gene){
      if(gene != false){
	    var elemRect = cumulativeOffset(thiz),
                offsetLeft = elemRect.left,
                offsetTop = elemRect.top + thiz.offsetHeight;
	    var select = divPlot.append("select")
	      .attr("name","r")
	      .attr("size", gene.length+1)
	      .style({"position":"absolute", "top": offsetTop+"px", "left": offsetLeft+"px"});
	    select.selectAll("option").data(gene)
              .enter().append("option")
                .property("value", function(d){ return d[0]+":"+d[1]+"-"+d[2]; })
                .text(function(d){ return (d[4])?(d[3]+" - "+d[4]):d[3]; })
                .on("click", function(){
                  displayTracks(d3.select(this).property("value"));
                })
      }
    });
  }
}

function searchGlobal(){
  var str = d3.select("input[name=r]").node().value,
      patt = new RegExp("^[\_\.0-9A-Za-z\-]{2,}$"),
      highlighter = function(s){
        return s.replace(new RegExp(str, 'gi'),function(o){ return '<span class="highlight">'+o+'</span>'; });
      }
  if(patt.test(str)){
      searchDB(str, "scores", function(res){
      if(res != false){
        showWindow('<ul id="search-results"></ul>');
        var li = d3.select("ul#search-results")
                  .selectAll("li").data(res)
                .enter().append("li")
        li.append("h5")
          .html(function(d){ return highlighter(d[3]); })
          .style({"cursor":"pointer","color":"#1f77b4","display":"inline"})
          .on("click", function(d){
            chr = d[0];
            start = d[1];
            end = d[2];
            d3.select("div.plot > div.window").remove()
            divPlot.selectAll("div.plot>*").style("display",null);
            displayTracks();
          })
        li.each(function(d){
          var dat = d[4]?d[4].split("|"):[];
          if(dat.length > 1){
            d3.select(this).append("ul")
              .style('padding','2px 0 8px 10px')
              .selectAll("li")
              .data(dat)
            .enter().append("li")
              .html(function(d){ return highlighter(d); })
          }
          if(dat.length == 1){
            d3.select(this).append("span")
              .html(" - " + highlighter(dat[0]))
          }
        })
      }else{
            showWindow("not found");
      }
    });
  }
}

function displaySelector(gChr, chrScale, selector){

  var brush = d3.svg.brush()
      .x(chrScale)
      .extent(selector?selector:[0,0])
      .on("brush", brushmove);

  var gBrush = gChr.select("g.brush")
  if(gBrush.empty()){

    gBrush = gChr.append("g")
      .attr("class","brush")
      .call(brush);

    gBrush.select(".background")
      .attr("height", 10)
      .style("cursor","pointer")
      .on("mousedown", !selector ? function(){
        var id = gChr.attr("id");
        d3.selectAll(".brush .extent, .brush .resize").style("display",function(){ return id==this.parentNode.parentNode.id?null:"none"; });
      } : null)

    gBrush.selectAll(".resize")
      .style("cursor","move")
      .append("rect")
        .attr("class", "handle")
        .attr("height", 10)
        .attr("width", 2)
        .style("fill","blue")

    gBrush.select(".extent")
      .attr("height", 10)
      .style({"fill-opacity": ".8", "stroke": "blue", "fill": "none"});

  }else{

    gBrush.call(brush);

  }

  function brushmove() {
    var extent = brush.extent(),
        bp = (typeof start != 'undefined')?end-start:1e+5;

    if(bp>chrScale.domain()[1]){
      start = 0;
      end = chrScale.domain()[1];
      gBrush.call(brush.extent([start,end]));
    }else{
      start = Math.round(d3.min(extent));
      end = Math.round(d3.max(extent));
      if(end-start==0){
        end = start+(bp/2);
        start = start-(bp/2);
        if(start<0){
          start = 0;
          end = bp;
        }
        if(end>chrScale.domain()[1]){
          start = chrScale.domain()[1]-bp;
          end = chrScale.domain()[1];
        }
        gBrush.call(brush.extent([start,end]));
      }
    }
      
    if(typeof selector != 'object'){
      d3.select("body a>button").attr("disabled", null)
        .on("click", function(){
          chr = gChr.attr("id");
          displayTracks();
        })
    }else{
      displayTracks();
    }
  }
}

function getTrackY(svg){
  try{
    var bbox = svg.node().getBBox(),
    trackY = bbox.y+bbox.height+20;

    return trackY;
  }catch(e){
    return -1;
  }
}

function resizeHeight(sel){
  var bbox = sel.node().getBBox();
  height =  Math.ceil(bbox.height);
  tHeight = height + margin.top + margin.bottom;
  d3.select(".plot>svg").attr("height", tHeight);
  d3.select("#mask rect").attr("height", height);
}

function addTitle(sel){
  sel.append("title")
    .text(function(d){return (d[5]?d[5]+"\n\n":"")+((d[6])?(d[6].split("|").join("\n")+"\n\n"):"")+d[2]+":"+(d[3]+1)+(((d[3]+1)==d[4])?"":("-"+d[4]));});
}

function addVCFdata(sel){
  sel.style("cursor","pointer")
     .on("click",function(d){
    showWindow((d[5]?"<h4>"+d[5]+"</h4>":"")+((d[6])?"<ul><li>"+(d[6].split("|").join("</li><li>")+"</li></ul>"):"")+"<p>"+d[2]+":"+(d[3]+1)+(((d[3]+1)==d[4])?"":("-"+d[4]))+"</p>");
  })
}

var xhrGetTracks;
function getTracks(chr, start, end, callback){
    if(typeof xhrGetTracks != 'undefined')
      xhrGetTracks.abort();
    xhrGetTracks = d3.json(queryUrl)
	    .header("Content-type", "application/x-www-form-urlencoded")
	    .post("action=query&gb="+gbID+"&r="+encodeURI(chr+":"+start+"-"+end), function(error, json) {
    if (error) throw error;
	    callback(json);
        });
}

function genomebrowser(){

  margin = {top: 80, right: 50, bottom: 0, left: 50};
  width = tWidth - margin.left - margin.right;
  height = tHeight - margin.top - margin.bottom;

  divPlot.selectAll("*").remove();

  var nav = divPlot
    .style({"width": tWidth+"px", "margin": "0 auto"})
    .append("div")
      .style({"padding-left": margin.left+"px", "padding-right": margin.right+"px", "margin-top": "20px"})
  nav.append("span")
       .attr("class","location")
  nav.append("input")
     .attr("type", "text")
     .attr("name", "r")
     .style("width","300px")
     .on("keyup", function(){
        if (d3.event.keyCode == 13) { gotoLocation(); }
	else { searchGene(); }
      })
     .on("paste", function(){
	searchGene();
      })
  nav.append("button")
     .text("Go to")
     .on("click", gotoLocation);
  nav.append("button")
     .text("Search")
     .on("click", searchGlobal);

  imgInfo(nav, function(sel){
    searchDB("random", "random", function(gene) {
      var example = gene[0];
      sel.append("style").text(".window p { margin: 20px 0; } .window p>svg { vertical-align: middle; }")
      sel.append("p").html("Input a region specified like '"+(example[0]+":"+example[1]+"-"+example[2])+"' or '"+(example[3])+"', then click <button>Go to</button>")
      sel.append("p").html("Input a word to search in browser, then click <button>Search</button>")
      sel.append("p").html(infoImages.clickHide+" Click on a track's name to hide/show it.")
      sel.append("p").html("Drag n' drop (horizontal) or right and left arrows to advance in the browser.")
      sel.append("p").html("Drag n' drop (vertical) to reorder tracks.<span style=\"padding: 0 4%;\"></span>"+infoImages.move)
      sel.append("p").html(infoImages.rescale+"Click on a value track's scale to change it.")
      sel.append("p").html("Click on genes, exons, domains or vcf will display a window with information about the feature.")
      sel.append("p").html("Hover the features for information.<span style=\"padding: 0 4%;\"></span>"+infoImages.clickInfo)
      sel.append("p").html("In vcf's information window, hover the acronyms for explanation.")
    });
  });

  if(dataTracks.map(function(d){ return d[2]; }).indexOf("value")!=-1){
    nav.append("button")
      .attr("class","bounds")
      .text("BOUNDS")
      .attr("title","set fixed boundaries for 'value' tracks")
      .style("float","right")
      .on("click", function(){
        fixedBounds = !fixedBounds;
        d3.select(this).style("background",fixedBounds?"#aaa":null)
        displayTracks();
      });
  }

  nav.append("button")
    .text("MAP")
    .attr("title","go back to genomemap")
    .style("float","right")
    .on("click", function(){
      divPlot.selectAll("*").remove()
      genomemap();
    });

  nav.append("button")
    .text("FASTA")
    .attr("title","get FASTA of this region")
    .style("float","right")
    .on("click", function(){
	  window.open("?r="+chr+":"+start+"-"+end, "_blank");
    });

var svg = divPlot.append("svg")
    .attr("width", tWidth)
    .attr("height", tHeight);

  svg.append("style")
    .text("text { font: 10px sans-serif; } "+
".segment text { font: 8px monospace; } "+
".axis path, .axis line { fill: none; stroke: #000; shape-rendering: crispEdges; } "+
".axis text, text.trackName { fill: #333; }"+
"text.trackName { cursor: pointer; }"+
".buttons {fill: grey;}" +
".bands>* {stroke: #000;  fill: #FFF;} "+
".acen {fill: #DDD;} "+
".gpos25 {fill: rgb(25%, 25%, 25%);} "+
".gpos33 {fill: rgb(33%, 33%, 33%);} "+
".gpos50 {fill: rgb(50%, 50%, 50%);} "+
".gpos66 {fill: rgb(66%, 66%, 66%);} "+
".gpos75 {fill: rgb(75%, 75%, 75%);} "+
".gpos100 {fill: #000;} "+
".gneg {fill: #FFF;} "+
".gvar {fill: #CCC;} "+
".stalk {fill: #666; stroke: none}");

  svg.append("defs")
    .append("clipPath")
    .attr("id", "mask")
    .append("rect")
        .attr("x", 0)
        .attr("y", 0)
        .attr("width", width)
        .attr("height", height);

  var buttons = svg.append("g")
      .attr("class", "buttons")
      .attr("transform", "translate("+margin.left+","+(margin.top-64)+")")

  buttons.append("polygon")
      .attr("points", "0,5 5,0 5,5 10,0 10,10 5,5 5,10")
  buttons.append("circle")
      .attr("cx","6")
      .attr("cy","5")
      .attr("r","7")
      .style("opacity", "0.2")
      .on("click", function(){
          var move = end-start;
          start = start-move;
          end = end-move;
          displayTracks();
        })

  buttons.append("polygon")
      .attr("points", "0,5 10,0 10,10")
      .attr("transform", "translate("+15+",0)")
  buttons.append("circle")
      .attr("transform", "translate("+15+",0)")
      .attr("cx","6")
      .attr("cy","5")
      .attr("r","7")
      .style("opacity", "0.2")
      .on("click", function(){
          var move = parseInt((end-start)*0.1);
          start = start-move;
          end = end-move;
          displayTracks();
        })

  buttons.append("rect")
      .attr("transform", "translate("+(width/2-20)+",0)")
      .attr("x","0")
      .attr("y","3.5")
      .attr("width","10")
      .attr("height","3")
  buttons.append("circle")
      .attr("transform", "translate("+(width/2-20)+",0)")
      .attr("cx","5")
      .attr("cy","5")
      .attr("r","7")
      .style("opacity", "0.2")
      .on("click", function(){
          var bp = end-start,
              center = start+parseInt(bp/2),
              zoomout = parseInt(bp/0.8);
          start = center-zoomout/2;
          end = center+zoomout/2;
          displayTracks();
        })

  buttons.append("path")
      .attr("d", "M 3.6308594 0 A 3.6302359 3.6302359 0 0 0 0 3.6308594 A 3.6302359 3.6302359 0 0 0 3.6308594 7.2597656 A 3.6302359 3.6302359 0 0 0 5.4355469 6.7753906 L 8.6601562 10 L 10 8.6601562 L 6.7753906 5.4355469 A 3.6302359 3.6302359 0 0 0 7.2597656 3.6308594 A 3.6302359 3.6302359 0 0 0 3.6308594 0 z M 3.6308594 0.90820312 A 2.6832178 2.6832178 0 0 1 6.3125 3.5917969 A 2.6832178 2.6832178 0 0 1 3.6308594 6.2753906 A 2.6832178 2.6832178 0 0 1 0.94726562 3.5917969 A 2.6832178 2.6832178 0 0 1 3.6308594 0.90820312 z ")
      .attr("transform", "translate("+(width/2-5)+",0)")

  buttons.append("path")
      .attr("d", "M 3.5 0 L 3.5 3.5 L 0 3.5 L 0 6.5 L 3.5 6.5 L 3.5 10 L 6.5 10 L 6.5 6.5 L 10 6.5 L 10 3.5 L 6.5 3.5 L 6.5 0 L 3.5 0 z ")
      .attr("transform", "translate("+(width/2+10)+",0)")
  buttons.append("circle")
      .attr("transform", "translate("+(width/2+10)+",0)")
      .attr("cx","5")
      .attr("cy","5")
      .attr("r","7")
      .style("opacity", "0.2")
      .on("click", function(){
          var bp = end-start,
              center = start+parseInt(bp/2),
              zoomin = parseInt(bp*0.8);
	  if(zoomin<100)
	    zoomin = 100;
          start = center-zoomin/2;
          end = center+zoomin/2;
          displayTracks();
        })

  buttons.append("polygon")
      .attr("points", "0,5 -10,0 -10,10")
      .attr("transform", "translate("+(width-15)+",0)")
  buttons.append("circle")
      .attr("transform", "translate("+(width-15)+",0)")
      .attr("cx","-6")
      .attr("cy","5")
      .attr("r","7")
      .style("opacity", "0.2")
      .on("click", function(){
          var move = parseInt((end-start)*0.1);
          start = start+move;
          end = end+move;
          displayTracks();
        })

 buttons.append("polygon")
      .attr("points", "0,5 -5,0 -5,5 -10,0 -10,10 -5,5 -5,10")
      .attr("transform", "translate("+width+",0)")
  buttons.append("circle")
      .attr("transform", "translate("+width+",0)")
      .attr("cx","-6")
      .attr("cy","5")
      .attr("r","7")
      .style("opacity", "0.2")
      .on("click", function(){
          var move = end-start;
          start = start+move;
          end = end+move;
          displayTracks();
        })

  svg = svg.append("g")
      .attr("class", "tracks")
      .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

  svg.append("g")
      .attr("class","chromosome")
      .attr("transform", "translate(0," + (-40) + ")");

  var tracks = svg.append("g")
      .attr("clip-path", "url(#mask)");

  svg.append("g")
      .attr("class", "x axis");

  var track = tracks.selectAll(".track")
      .data(dataTracks)
    .enter().append("g")
      .attr("class", function(d){ return "track "+d[2]; })
      .style("fill", function(d){ return d[3]; })
      .property("scale", function(d){ return d[2]=="score"?JSON.parse(d[4]):null; })


  track.append("rect")
          .attr("class","rectBack")
          .attr("x",0)
          .attr("y",-20)
          .attr("width",width)
          .attr("height",0)

  track.append("text")
        .attr("class","trackName")
        .text(function(d){ return d[1]; })
        .attr("y", -6)
        .attr("x", 6)
        .on("click",function(d){
          d[5] = d[5]?0:1;
          displayTracks();
        })

  var initX;
  var widthScale = d3.scale.linear()
        .domain([0,width]);
  var sortTracks = function(a,b){
    a = a[6];
    b = b[6];
    return a < b ? -1 : a > b ? 1 : a >= b ? 0 : NaN;
  }

  var drag = d3.behavior.drag()
             .on('dragstart', function(){
               tracks.style("cursor","move");
               initX = d3.mouse(tracks.node())[0];
             })
             .on('drag', function(thisData) {
               var coor = d3.mouse(tracks.node())[1];
               if(coor%10==0){
                 track.each(function(d){
                     if(coor > d[6] && coor < d[6]+d[7]){
                       var idY = thisData[6];
                       thisData[6] = d[6];
                       d[6] = idY;
                       track.sort(sortTracks)
                         .style("display","none")
                         .each(function(d,i){
                           d[6] = getTrackY(tracks);
                           d3.select(this)
                               .attr("transform","translate(0,"+d[6]+")")
                               .style("display",null)
                               .select(".rectBack")
                                 .style("fill",i%2==1?"#f0f0f0":"#fcfcfc")
                         })
                     }
                 });
               }
             })
             .on('dragend', function() {
               tracks.style("cursor",null);
               var coor = d3.mouse(tracks.node())[0],
                   x = coor - initX;
               if(Math.abs(x)>20){
                 widthScale.range([0,end-start]);
                 var move = Math.round(widthScale(x));
                 start = start - move;
                 end = end - move;
                 displayTracks();
               }
             });

  track.call(drag)

  return svg;
}

function decodeLocation(strLocation){
  chr = strLocation.split(":")[0];
  start = parseInt(strLocation.split(":")[1].split("-")[0]);
  end = parseInt(strLocation.split(":")[1].split("-")[1]);

  if(isNaN(end)){
    if(start>50){
      start = start-50;
      end = start+100;
    }else{
      start = 0;
      end = 100;
    }
  }
}

function displayTracks(strLocation){

  if(strLocation)
    decodeLocation(strLocation);

  var svg = d3.select(".plot>svg g.tracks");
  if(svg.empty())
    svg = genomebrowser();

  var bp = end-start;
  if(bp<100){
    end = start+100;
    bp = 100;
  }

  if(start<0){
    start = 0;
    end = bp;
  }

  d3.json(queryUrl)
	    .header("Content-type", "application/x-www-form-urlencoded")
	    .post("action=query&gb="+gbID+"&chromosome="+encodeURI(chr.toLowerCase()), function(error, json) {
    if (error) throw error;

    var chrEnd = d3.max(json, function(d){
      return +d[1];
    });

    if(end>chrEnd){
      end = chrEnd;
      start = chrEnd-bp;
      if(start<0)
        start = 0;
    }

var chrScale = d3.scale.linear()
    .range([0, width])
    .domain([0, chrEnd]);

var x = d3.scale.linear()
    .range([0, width])
    .domain([start, end]);

var scoreScale = d3.scale.linear()
    .clamp(true);

var valueScale = d3.scale.linear()
    .range([50, 0])
    .clamp(true);

var displayValueTrack = function(track){
  x.domain([start,end]);
  valueScale.domain(track.property("scale")).nice();
  var vAxis = d3.svg.axis()
	.scale(valueScale)
	.tickValues(valueScale.domain())
	.orient("left");
  track.selectAll("g.values>path")
        .attr("d", function(d){
          var s = x(d[3]),
              e = x(d[4]),
              h = valueScale(d[6]),
              b = valueScale(0);
          return "M"+s+","+h+"L"+e+","+h+"L"+e+","+b+"L"+s+","+b+"Z"; }
        );
  track.select(".axis").call(vAxis);
}

var cutSegmentStart = function(x){
  var bound = -margin.left;
  return ((x<bound)?bound:x)
}

var cutSegmentEnd = function(x){
  var bound = width+margin.right;
  return ((x>bound)?bound:x)
}

var xAxis = d3.svg.axis()
    .scale(x)
    .orient("top")
    .tickFormat(function(d){
      if(d>1e+6)
        return d3.round((d/1e+6),2)+"M";
      if(d>1e+3)
        return d3.round((d/1e+3),2)+"K";
      return d;
    });

  d3.select("span.location")
      .text("location "+chr+":"+start+"-"+end+" "+(end-start)+"bp ")

  d3.select("input[name=r]")
      .property("value","");

  if(typeof displayChr == 'function'){
    var gChr = svg.select("g.chromosome").attr("id", chr);
    displayChr(gChr, chrScale, json);
    displaySelector(gChr, chrScale, [start, end]);
  }

  svg.select(".x.axis")
      .call(xAxis);

  var gTracks = svg.select("g[clip-path]")

  var gTrack = gTracks.selectAll(".track")
        .style("display","none");

  getTracks(chr,start,end,function(allData){

    gTrack.each(function(d,trackIndex){

      var trackID = +d[0], 
      type = d[2],
      hiddenTrack = d[5],
      trackY = d[6] = getTrackY(gTracks);

      var data = allData['track'+trackID];

      var track = d3.select(this)
        .attr("transform", "translate(0,"+trackY+")")
        .style("display",null);

      if(!data||data[0][2]==false||hiddenTrack){
        track.selectAll(":not(.rectBack):not(.noData):not(.trackName)").remove();
        var noData = track.selectAll(".noData")
          .data([hiddenTrack?"(hidden track)":(data?"(zoom in to view this track)":"(empty region)")])
        noData.enter().append("text")
          .attr("class","noData")
          .attr("transform", "translate("+(width/2)+",0)")
          .attr("text-anchor", "middle")
          .style("fill","#333");
        noData.text(String);

      }else{

      track.select(".noData").remove();

      switch(type){
        case "value":

          if(fixedBounds){
            if(this.newscale)
              this.scale = this.newscale;
            else
              this.scale = JSON.parse(d[4]);
          }else{
            delete this.newscale;
            var minval = d3.min(data,function(d){ return +d[6]; }),
                maxval = d3.max(data,function(d){ return +d[6]; });
            if(minval>0)
              minval = 0;
            if(maxval<0)
              maxval = 0;
            this.scale = [minval,maxval];
          }

          track.selectAll(".track>g")
            .data(["values","axis"])
            .enter().append("g")
            .attr("class",String)
            .each(function(d){
              if(d=="axis"){
                d3.select(this).attr("transform", "translate("+(width-1)+")")
                .on("click", function(){
                    var win = showWindow();
                    win.append("p")
		    win.append("p")
			.append("input")
			  .attr("type","number")
			  .attr("name","max")
			  .style("width", "50px")
			  .property("value",this.childNodes[1].textContent.replace(",",""))
		    win.append("p")
			.append("input")
			  .attr("type","number")
			  .attr("name","min")
			  .style("width", "50px")
			  .property("value",this.childNodes[0].textContent.replace(",",""))
		    win.append("p")
			.append("input")
			  .attr("type", "button")
			  .attr("name", "setScale")
			  .style("width", "50px")
			  .property("value", "set")
			  .on("click", function(){
			    fixedBounds = true;
			    d3.select("button.bounds").style("background","#aaa")
			    var newScale = [d3.select("input[name=min]").property("value"),d3.select("input[name=max]").property("value")];
			    track.property("newscale",newScale);
			    track.property("scale",newScale);
			    displayValueTrack(track);
			  })
                });
              }
            });

          var values = track.select(".values").selectAll("path")
                .data(data, function(d){ return d[0]; })
          values.enter().append("path").call(addTitle);
          values.exit().remove();

          displayValueTrack(track);

        break;
        case "score":

          scoreScale.domain(track.property("scale")).nice();

          var scores = track.selectAll("path")
              .data(data, function(d){ return d[0]; })
          scores.enter().append("path")
            .style("fill-opacity",function(d){ return scoreScale(d[6]); })
            .call(addTitle);
          scores.exit().remove();
          scores.attr("d",function(d){ 
                return "M"+x(d[3])+",0L"+x(d[4])+",0L"+x(d[4])+",10L"+x(d[3])+",10Z";
              })

        break;
        case "vcf":

          var csqFormat = [],
              csqColor = d3.scale.ordinal()
                .range(["#ff7f0e","#2ca02c","#9467bd","#8c564b","#e377c2","#7f7f7f","#bcbd22","#17becf"]);

          if(d[4])
            csqFormat = JSON.parse(d[4]);

          track.selectAll(".legend")
            .data(csqFormat)
              .enter().append("text")
            .attr("class","legend")
            .attr("x",6)
            .attr("y",function(d,i){ return 22+(12*i); })
            .style("font-size", "8px")
            .text(String)

          var segment = track.selectAll(".segment")
            .data(data, function(d){ return d[0]; })

          var smgEnter = segment.enter().append("g")
            .attr("class", "segment")

          segment.exit().remove();

          smgEnter.each(function(d){
            var thiz = d3.select(this),
                attrs = d[6].split("|"),
                aux = [],
                score = {};

            attrs.forEach(function(p){
              aux = p.split("=");
              score[aux[0]] = aux[1];
            })

            thiz.append("path")
              .attr("d","M-1,0L2,0L2,10L-1,10Z")
              .style("fill","steelblue")

            if(score['AF']!=null){
              thiz.append("path")
                .attr("d",function(d){
                  var h = 10*(1-score['AF']);
                  return "M-1,"+h+"L2,"+h+"L2,10L-1,10Z";
                })
                .style("fill","firebrick")
            }

            if(csqFormat.length){
              var h = 22,
                val = "";
              csqFormat.forEach(function(i){
                  val = score[i];
                  if(val){
                    thiz.append('rect')
                      .attr("x",-3)
                      .attr("height",10)
                      .attr("y",h-8)
                      .attr("width",7)
                      .attr('rx',2)
                      .style('fill',function(){
                        var patt = new RegExp("_MAF$");
                        if(patt.test(i) || i=="GMAF"){
                          var n = parseFloat(val.split(":")[1])
                          return d3.lab((1-n)*100,0,-40).toString();
                        }
                        if(i=="IMPACT"){
                          return val=="HIGH"?"#a50f15":(val=="MODERATE"?"#de2d26":(val=="LOW"?"#fb6a4a":"#fc9272"));
                        }
                        return csqColor(i);
                      })
                    thiz.append('text')
                      .attr("x",-2)
                      .attr("y",h)
                      .style("fill","#ffffff")
                      .text(val.slice(0,1).toUpperCase())
                      .append("title")
                        .text(i+'='+val)
                  }
                  h = h + 12;
              });
            }
          })

          smgEnter.call(addVCFdata);

          segment.attr("transform",function(d){ return "translate("+x(d[3])+",0)"; })

        break;
        case "vcfsample":

          var segment = track.selectAll(".segment")
            .data(data, function(d){ return d[0]; })

          var smgEnter = segment.enter().append("g")
            .attr("class", "segment")

          segment.exit().remove();

          smgEnter.append("path")
            .attr("d","M-0.5,0L2,0L2,10L-0.5,10Z")
            .style("fill","steelblue")

          smgEnter.append("path")
            .attr("d",function(d){
              var h = 10,
                  attr = d[6].split("|"),
                  AD = attr.filter(function(d){ return d.indexOf("AD=")==0; }),
                  RO = 1,
                  AO = 0;
              if(AD.length==1){
                AD = AD[0].split("=")[1].split(",");
                RO = parseInt(AD[0]);
                AO = parseInt(AD[1]);
              }else{
                AD = [];
                AD[0] = attr.filter(function(d){ return d.indexOf("RO=")==0; }),
                AD[1] = attr.filter(function(d){ return d.indexOf("AO=")==0; });
                if(AD[0].length==1 && AD[1].length==1){
                  RO = parseInt(AD[0][0].split('=')[1]);
                  AO = parseInt(AD[1][0].split('=')[1]);
                }
              }
              h = h-(h/(RO+AO)*AO);
              return "M-0.5,"+h+"L2,"+h+"L2,10L-0.5,10Z";
            })
            .style("fill","firebrick")

          smgEnter.call(addVCFdata);

          segment.attr("transform",function(d){ return "translate("+x(d[3])+",0)"; })

        break;
        default:

          var segment = track.selectAll(".segment")
            .data(data, function(d){ return d[0]; })

          var smgEnter = segment.enter().append("g")
            .attr("class", "segment")

          segment.exit().remove();

          smgEnter.append("path");
	  smgEnter.append("text")
	      .attr("y", 20)
	      .text(function(d){
		     var name = d[5]?d[5]:"";
		     if (d[7]=="+")
		       return name+">";
		     if(d[7]=="-")
		       return "<"+name;
		     return name;
		   });
	  smgEnter.call(addTitle);

          segment.selectAll("text")
	    .attr("x", function(d){ return x(d[3]); })

          segment.selectAll("path")
            .attr("d", function(d){
	      var sgmStart = cutSegmentStart(x(d[3])), sgmEnd = cutSegmentEnd(x(d[4]));

	      if(type == "domain")
	        return "M"+sgmStart+",0L"+(sgmStart+1)+",0L"+(sgmStart+1)+",4L"+(sgmEnd-1)+",4L"+(sgmEnd-1)+",0L"+sgmEnd+",0L"+sgmEnd+",10L"+(sgmEnd-1)+",10L"+(sgmEnd-1)+",6L"+(sgmStart+1)+",6L"+(sgmStart+1)+",10L"+sgmStart+",10Z";

	      if(type == "exons" && d[11]){
                var i;
                for(i = 12; i<=13; i++)
                  if(d[i].slice(-1)==",")
                    d[i] = d[i].slice(0,-1);
	        var dAttr = "M"+sgmStart+",6L"+sgmStart+",4L",
                    blockSizes = d[12].split(","),
                    blockStarts = d[13].split(",");
	        for(i = 0; i<d[11]; i++){
	          var exonStart = x(d[3]+parseInt(blockStarts[i])), exonEnd = x(d[3]+parseInt(blockStarts[i])+parseInt(blockSizes[i]));
	          if(exonEnd>0 && exonStart<width){
	            dAttr += exonStart + ",4L"+exonStart+",0L"+exonEnd+",0L"+exonEnd+",10L"+exonStart+",10L"+exonStart+",4L";
	          }
	        }
	        dAttr += sgmEnd+",4L"+sgmEnd+",6Z";
	        return dAttr;
	      }

	      return "M"+sgmStart+",0L"+sgmEnd+",0L"+sgmEnd+",10L"+sgmStart+",10Z";
            })

	    smgEnter.style("cursor","pointer")
              .on("click", function(d){
	        window.open("?r="+d[2]+":"+d[3]+"-"+d[4]+"&n="+d[5]+(d[6]?"&score="+encodeURI(d[6]):"")+((d[7]=="-")?"&strand=reverse":"")+((d[8]||d[9])?"&thickStart="+d[8]+"&thickEnd="+d[9]:"")+(d[11]?"&blockCount="+d[11]+"&blockSizes="+d[12]+"&blockStarts="+d[13]:""), "_blank");
	      })

          var lines = [];
          lines.push(-Infinity);
          segment.attr("transform",function(d){
            var selfCoords = this.getBBox(),
            i = 0;
            while(lines[i]>=selfCoords.x){
              i++;
              if(typeof lines[i] == 'undefined')
                lines.push(-Infinity);
            }
            lines[i] = selfCoords.x+selfCoords.width;
            return "translate(0,"+(i*24)+")";
          });
      }
      }
      var rectBack = track.select(".rectBack")
            .style("fill", function(){ return trackIndex%2==1?"#f0f0f0":"#fcfcfc"; })
            .attr("height",0),
          trackBBox = track.node().getBBox(),
          trackHeight = d[7] = trackBBox.y+trackBBox.height+30;
      rectBack.attr("height",trackHeight);
    });
    resizeHeight(svg);
  });
});
}

displayGenomeBrowser();
